#ifndef DV_TICKET_AUTHENTICATOR
#define DV_TICKET_AUTHENTICATOR
// $Id: authenticator.h,v 1.9 2003/07/23 17:58:38 dvermeir Exp $
#include <stdexcept>
#include <dvticket/user.h>

namespace Dv {
namespace Ticket {
/**
 * A class that can check a user's identity,
 * based on a login name and password.
 * This is an abstract class. An application should implement
 * a derived class. The default implementation always succeeds.
 *
 * @see Dv::Ticket::Request
 */
class Authenticator {
public:
  /** Destructor. */
  virtual ~Authenticator() {}
 
  /** @return name of this Authenticator.
   * Derived classes should have different names, which will
   * show up in the server logs.
   * @sa Dv::Ticket::Server
   */
  virtual std::string const name() { return "DefaultAuthenticator"; }

  /** Actual authentication of a user.
   * @param host from which user requests authentication
   * @param name login name of the user.
   * @param passwd password of the user.
   * @param info extra info on user.
   * @exception runtime_error if there is no user
   *   corresponding to the @a name and @a passwd.
   */
  virtual Dv::Ticket::User validate(const std::string& host, const std::string& name,
    const std::string& passwd, const std::string& info="") 
      throw (std::runtime_error) { return Dv::Ticket::User("student", 10, name, info); }

  /** Return an anonymous user. Derived classes may decide, based on
   * the various parameters whether they will allow an anonymous
   * ticket to be issued.
   * @param host from which user requests authentication (dot address)
   * @param name login name of the user.
   * @param info extra info on user.
   * @exception runtime_error if the operation failed.
   */
  virtual Dv::Ticket::User anon_validate(const std::string& host, const std::string& name,
    const std::string& info="") throw (std::runtime_error) {
      return Dv::Ticket::User("anonymous", 0, name, info); }
};

}}
#endif

