#ifndef	DVTICKET_USER_H
#define	DVTICKET_USER_H
// $Id: user.h,v 1.9 2003/08/06 12:12:06 dvermeir Exp $

#include <string>
#include <iostream>
#include <stdexcept>
#include <dvxml/xml.h>
#include <dvticket/exception.h>

namespace Dv { 
namespace Ticket {

class Request;
class Authenticator;
class Ticket;
/**
 * A User class for use by Dv::Ticket::Ticket.
 * A user is identified by a numeric @a id, and a  @a category.
 * A user also has a @a name and an uninterpreted @a info string. 
 *
 * The ticket database stores all components of the user-owner
 * of the ticket.
 * @sa Dv::Ticket::Ticket
 */
class User {
friend class Dv::Ticket::Request;
friend class Dv::Ticket::Authenticator;
friend class Dv::Ticket::Ticket;
public:
  /** @return id of this user.  */
  int id() const { return id_; }
  /** @return name of this user.  */
  const std::string& name() const { return name_; }
  /** @return category of this user.  */
  const std::string& category() const { return category_; }
  /** @return extra info on this user.  */
  const std::string& info() const { return info_; }

  /** @return true iff users have same id and category
   * @param user to compare this user with
   * @warning name is not taken into account!
   */
  bool operator==(const User& user) const { 
    return id() == user.id() && category() == user.category();
    }
  /** Comparison, first on category, then on id.
   * @return true iff this user is smaller than user
   * @warning name is not taken into account!
   */
  bool operator<(const User& user) const;

  /** Create XML representation of user.
   * @return xml::node representing user
   * @code
   * <user>
   *  <id>123</id>
   *  <category>student</category>
   *  <name>fred</name>
   *  <info>Jan Janssens</info>
   * </user>
   * @endcode
   */
  Dv::Xml::Node xml(const std::string& name="user") const;
   
  /** Constructor.
   * @param category of user.
   * @param id of user.
   * @param name of user.
   * @param info uninterpreted extra info
   */
  User(const std::string& category="anonymous", int id=0,
    const std::string& name="anonymous", const std::string& info =""): id_(id),
      category_(category), name_(name), info_(info) {}

  /** Constructor.
   * @param xml_user XML representation of User
   * @exception Dv::Ticket::Exception if anything goes wrong
   * @sa Dv::Ticket::User::xml
   */
  User(Dv::Xml::Node::Ref xml_user) throw (Dv::Ticket::Exception);
private:

  /** Id of user, should be unique within the same category. */
  int  id_;
  /** Category of user. */
  std::string  category_;
  /** User name (e.g. login name). */
  std::string  name_;
  /** Extra info */
  std::string  info_;
};

} }
#endif
